% File src/library/base/man/unique.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2014 R Core Team
% Distributed under GPL 2 or later

\name{unique}
\alias{unique}
\alias{unique.default}
\alias{unique.data.frame}
\alias{unique.matrix}
\alias{unique.array}
\title{Extract Unique Elements}
\description{
  \code{unique} は \code{x} のようなベクトル，データフレーム又は配列
  を返すが，重複した要素/行は除かれる．
}
\usage{
unique(x, incomparables = FALSE, \dots)

\method{unique}{default}(x, incomparables = FALSE, fromLast = FALSE,
        nmax = NA, \dots)

\method{unique}{matrix}(x, incomparables = FALSE, MARGIN = 1,
       fromLast = FALSE, \dots)

\method{unique}{array}(x, incomparables = FALSE, MARGIN = 1,
       fromLast = FALSE, \dots)
}
\arguments{
  \item{x}{ベクトル，データフレーム，配列，又は \code{NULL}．}
  \item{incomparables}{比較不可能な値のベクトル．
    \code{FALSE} は特殊値で，全ての値が比較可能なことを意味し，
    恐らく既定以外のメソッドに対して受け入れられる唯一の値である．
    これは \code{x} と同じ型に内部的に強制変換される．}
  \item{fromLast}{重複は最後から考慮すべきかどうかを指示する論理値．
    つまり同一の値の最後(又は最右翼)を残す．
    これは \code{\link{names}} や \code{\link{dimnames}} に対してだけ
    問題になる．}
  \item{nmax}{期待されるユニークな項目の最大値(1 以上)．
    \code{\link{duplicated}} を見よ．}
  \item{\dots}{特別なメソッドに対する引数．}
  \item{MARGIN}{固定される配列マージン：単一の整数．}
}
\details{
  これは総称的関数でありベクトル，データフレームそして配列に対する
  メソッドをもつ(行列を含む)．

  配列メソッドは \code{MARGIN} で指定される次元の各要素に対して残りの
  次元が前の要素(行主導の順序で)と同一であるかどうかを計算する．
  これは最も普通にはユニークな行(既定)や列(\code{MARGIN = 2} で)を見つけるために
  使われる．

  Unix 命令 \code{uniq} とは異なり，これは\emph{重複した}要素/行を
  除き単に \emph{繰り返される}要素/行ではない．つまり，ある要素は
  もしそれが任意の先にある要素に等しければ除き，単にそれが直前のものに
  等しい等しい場合だけではない．(後者の場合については
  \code{\link{rle}} を見よ)．

  欠損値は等しいと見做されるが，\code{NaN} は \code{NA_real_} に等しくない．
  文字列はもしそれらが異なったエンコーディングを持つが UTF-8 に翻訳された時
  一致すれば等しいと見做される．

  \code{incomparables} 中の値は決して重複するとマークされない．
  これはかなり小さな値のセットに対して使われることを念頭に置いており，
  非常に大きなセットに対しては効率的ではない．

  一つ以上の列を持つデータフレーム，又は1以上の長さの次元を比較する
  時の配列や行列に対して使われると，これは文字列表現の同等性をテストする．
  これは浮動点小数の正確な同等性を無思慮に信頼する人々を発見する!

  文字列はもし入力のどれかが \code{"bytes"} (\code{\link{Encoding}} を見よ)と
  マークされていればバイト列として比較される．
}
\value{
  ベクトルに対しては \code{x} と同じ型のオブジェクトであるが，
  重複した要素の各々に対して唯一つのコピーを持つ．
  属性はコピーされない(従って結果は名前を持たない)．

  データフレームに対しては，同じ列だが可能性としてより少ない
  行を持つデータフレームが返される(行名はユニークな行の最初に登場したものの
  行名を持つ)．

  行列や配列は \code{[, drop = FALSE]} によって部分抽出されるので，
  次元や次元名は適切にコピーされるが，
  結果は常に \code{x} と同じ数の次元を持つ．
}
\section{警告}{
  これをリストに使うと遅い可能性がある．特に要素が原子的でない(\code{\link{vector}} を見よ)
  場合や属性だけが異なる場合がそうである．最悪のケースの計算量は \eqn{O(n^2)} になる．
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
}
\seealso{
  重複した要素の添字を与える \code{\link{duplicated}}．

  Unix の命令  \code{uniq -c} に同値な \code{\link{rle}}．
}
\examples{
x <- c(3:5, 11:8, 8 + 0:5)
(ux <- unique(x))
(u2 <- unique(x, fromLast = TRUE)) # 異なった順序
stopifnot(identical(sort(ux), sort(u2)))

length(unique(sample(100, 100, replace = TRUE)))
## 近似的に 100(1 - 1/e) = 63.21

unique(iris)
}
\keyword{manip}
\keyword{logic}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/unique.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2014 R Core Team
%% Distributed under GPL 2 or later
%
%\name{unique}
%\alias{unique}
%\alias{unique.default}
%\alias{unique.data.frame}
%\alias{unique.matrix}
%\alias{unique.array}
%\title{Extract Unique Elements}
%\description{
%  \code{unique} returns a vector, data frame or array like \code{x}
%  but with duplicate elements/rows removed.
%}
%\usage{
%unique(x, incomparables = FALSE, \dots)
%
%\method{unique}{default}(x, incomparables = FALSE, fromLast = FALSE,
%        nmax = NA, \dots)
%
%\method{unique}{matrix}(x, incomparables = FALSE, MARGIN = 1,
%       fromLast = FALSE, \dots)
%
%\method{unique}{array}(x, incomparables = FALSE, MARGIN = 1,
%       fromLast = FALSE, \dots)
%}
%\arguments{
%  \item{x}{a vector or a data frame or an array or \code{NULL}.}
%  \item{incomparables}{a vector of values that cannot be compared.
%    \code{FALSE} is a special value, meaning that all values can be
%    compared, and may be the only value accepted for methods other than
%    the default.  It will be coerced internally to the same type as
%       \code{x}.}
%  \item{fromLast}{logical indicating if duplication should be considered
%    from the last, i.e., the last (or rightmost) of identical elements will
%    be kept.  This only matters for \code{\link{names}} or
%    \code{\link{dimnames}}.}
%  \item{nmax}{the maximum number of unique items expected (greater than one).
%    See \code{\link{duplicated}}.}
%  \item{\dots}{arguments for particular methods.}
%  \item{MARGIN}{the array margin to be held fixed: a single integer.}
%}
%\details{
%  This is a generic function with methods for vectors, data frames and
%  arrays (including matrices).
%
%  The array method calculates for each element of the dimension
%  specified by \code{MARGIN} if the remaining dimensions are identical
%  to those for an earlier element (in row-major order).  This would most
%  commonly be used for matrices to find unique rows (the default) or columns
%  (with \code{MARGIN = 2}).
%
%  Note that unlike the Unix command \code{uniq} this omits
%  \emph{duplicated} and not just \emph{repeated} elements/rows.  That
%  is, an element is omitted if it is equal to any previous element and
%  not just if it is equal the immediately previous one.  (For the
%  latter, see \code{\link{rle}}).
%
%  Missing values are regarded as equal, but \code{NaN} is not equal to
%  \code{NA_real_}.  Character strings are regarded as equal if they are
%  in different encodings but would agree when translated to UTF-8.
%
%  Values in \code{incomparables} will never be marked as duplicated.
%  This is intended to be used for a fairly small set of values and will
%  not be efficient for a very large set.
%
%  When used on a data frame with more than one column, or an array or
%  matrix when comparing dimensions of length greater than one, this
%  tests for identity of character representations.  This will
%  catch people who unwisely rely on exact equality of floating-point
%  numbers!
%
%  Character strings will be compared as byte sequences if any input is
%  marked as \code{"bytes"} (see \code{\link{Encoding}}).
%}
%\value{
%  For a vector, an object of the same type of \code{x}, but with only
%  one copy of each duplicated element.  No attributes are copied (so
%  the result has no names).
%
%  For a data frame, a data frame is returned with the same columns but
%  possibly fewer rows (and with row names from the first occurrences of
%  the unique rows).
%
%  A matrix or array is subsetted by \code{[, drop = FALSE]}, so
%  dimensions and dimnames are copied appropriately, and the result
%  always has the same number of dimensions as \code{x}.
%}
%\section{Warning}{
%  Using this for lists is potentially slow, especially if the elements
%  are not atomic vectors (see \code{\link{vector}}) or differ only
%  in their attributes.  In the worst case it is \eqn{O(n^2)}.
%}
%\references{
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%}
%\seealso{
%  \code{\link{duplicated}} which gives the indices of duplicated
%  elements.
%
%  \code{\link{rle}} which is the equivalent of the Unix \code{uniq -c}
%  command.
%}
%\examples{
%x <- c(3:5, 11:8, 8 + 0:5)
%(ux <- unique(x))
%(u2 <- unique(x, fromLast = TRUE)) # different order
%stopifnot(identical(sort(ux), sort(u2)))
%
%length(unique(sample(100, 100, replace = TRUE)))
%## approximately 100(1 - 1/e) = 63.21
%
%unique(iris)
%}
%\keyword{manip}
%\keyword{logic}
%\encoding{UTF8}

\encoding{UTF8}

