% File src/library/parallel/man/unix/pvec.Rd
% Part of the R package, https://www.R-project.org
% Copyright 2009-2014 R Core Team
% Distributed under GPL 2 or later

\name{pvec}
\alias{pvec}
\title{Parallelize a Vector Map Function using Forking}
\description{
  \code{pvec} は関数のベクトル要素への実行を
  ベクトルを分割し各部分を一つのコアに任せることで
  並列実行する．
  関数はベクトル化マップでなければならない，つまり
  それはベクトル入力を取り
  ベクトルの分割とは無関係に入力と正確に同じ長さの出力を作る．

  これはフォークに基づいており，従って
  \code{mc.cores = 1} でない限り Windows では利用できない．
}
\usage{
pvec(v, FUN, ..., mc.set.seed = TRUE, mc.silent = FALSE,
     mc.cores = getOption("mc.cores", 2L), mc.cleanup = TRUE)
}

\arguments{
  \item{v}{操作対象のベクトル．}
  \item{FUN}{ベクトルの各部分に呼び出される関数．}
  \item{\dots}{ベクトルの後に \code{FUN} に渡される追加の引数．}
  \item{mc.set.seed}{\code{\link{mcparallel}} を見よ．}
  \item{mc.silent}{もし \code{TRUE} に設定すると
    \file{stdout} への全ての出力は全てのフォークされた並列プロセスに対して
    抑制される(\file{stderr} は影響を受けない)．}
  \item{mc.cores}{使用するコア数，つまり
    最大幾つまでの子プロセスを同時に実行するか．
    少なくとも一つでなければならない．
    このオプションはもし設定されていれば
    環境変数 \env{MC_CORES} から初期化される．}
  \item{mc.cleanup}{
    \code{\link{mclapply}} 中のこの引数の説明を見よ．}
}

\details{
  \code{pvec} は \code{FUN(x, ...)} を並列化する，
  ここで \code{FUN} は \code{x} と同じ長さのベクトルを返す．
  \code{FUN} はまたピュアでなければならない(つまり副作用を持たない)．
  なぜなら副作用は並列プロセスからは収集できないからである．
  ベクトルはほぼ同じサイズのの副ベクトルに分割され，それぞれに
  \code{FUN} が実行される．
  原理的には必ずしもマップでない関数を使うことは可能であるが，
  分割は理論的には任意であるため解釈はケース依存である
  (そうした場合警告が与えられる)．

  \code{pvec} と \code{\link{mclapply}} の主要な違いは，
  \code{mclapply} は \code{FUN} を各要素に個別に適用する一方
  \code{pvec} は \code{c(FUN(x[1]), FUN(x[2]))} が
  \code{FUN(x[1:2])} に等しいことを仮定するので
  コア(またはもし少なければ要素)があるだけ多くの
  各々がベクトルの部分集合を処理する \code{FUN} の呼び出しに分割され
  ることである．これは \code{pvec} を \code{mclapply} より効率的にするが
  \code{FUN} に関する上記の仮定を必要とする．

  もし \code{mc.cores == 1} ならこれは現在のプロセスで
  \code{FUN(v, ...)} を評価する．
}

\value{
  計算の結果 -- 成功の場合それは
  \code{v} と同じ長さになる．もしエラーが起きるか
  関数がマップでなければ，結果は短かったり長かったりし，
  警告が出る．
}

\note{
  並列化の特性から，
  エラーは文字列として返され抹消された子プロセスは単に
  非存在データとして現れるので，
  エラー処理は通常の規則に従わない．
  従って正しいサイズかどうか確証するため結果の長さを
  チェックするのはユーザの責任である．
  \code{pvec} はもしそうしたことが起きれば
  そうした出力が意図的なものかどうか分からないので警告を出す．

  これを GUI フロントエンドと共に使うことが
  勧められないことについては \code{\link{mcfork}} を見よ．
}

\author{
  Simon Urbanek と R Core．

  パッケージ \CRANpkg{multicore} 由来．
}
\seealso{
  \code{\link{mcparallel}}, \code{\link{mclapply}},
  \code{\link{parLapply}}, \code{\link{clusterMap}}.
}
\examples{
x <- pvec(1:1000, sqrt)
stopifnot(all(x == sqrt(1:1000)))

\donttest{
# 比較的遅い操作として大きなデータセット中のデータ文字列を
# Unix 時間に変換する
# それで先ずランダムな日付を最初に作ろう
# (コアを二つ使う小さなテスト： options("mc.cores") を設定し
# 良い大きなスケールのテストのため N を増やす．)
N <- 1e5
dates <- sprintf('\%04d-\%02d-\%02d', as.integer(2000+rnorm(N)),
                 as.integer(runif(N, 1, 12)), as.integer(runif(N, 1, 28)))

system.time(a <- as.POSIXct(dates))

# しかし書式を指定するほうが速い
system.time(a <- as.POSIXct(dates, format = "\%Y-\%m-\%d"))

# pvec の方が速いはずだが，システムのオーバーヘッドは高くなり得る
system.time(b <- pvec(dates, as.POSIXct, format = "\%Y-\%m-\%d"))
stopifnot(all(a == b))

# これに mclapply を使うと相当遅くなる．なぜなら各値は
# as.POSIXct() を lapply(dates, as.POSIXct) として
# 個別に呼び出す必要があるから
system.time(c <- unlist(mclapply(dates, as.POSIXct,  format = "\%Y-\%m-\%d")))
stopifnot(all(a == c))
}}
\keyword{interface}

\encoding{UTF-8}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/parallel/man/unix/pvec.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 2009-2014 R Core Team
%% Distributed under GPL 2 or later
%
%\name{pvec}
%\alias{pvec}
%\title{Parallelize a Vector Map Function using Forking}
%\description{
%  \code{pvec} parellelizes the execution of a function on vector elements
%  by splitting the vector and submitting each part to one core. The
%  function must be a vectorized map, i.e.\sspace{}it takes a vector input and
%  creates a vector output of exactly the same length as the input which
%  doesn't depend on the partition of the vector.
%
%  It relies on forking and hence is not available on Windows unless
%  \code{mc.cores = 1}.
%}
%\usage{
%pvec(v, FUN, ..., mc.set.seed = TRUE, mc.silent = FALSE,
%     mc.cores = getOption("mc.cores", 2L), mc.cleanup = TRUE)
%}
%
%\arguments{
%  \item{v}{vector to operate on}
%  \item{FUN}{function to call on each part of the vector}
%  \item{\dots}{any further arguments passed to \code{FUN} after the vector}
%  \item{mc.set.seed}{See \code{\link{mcparallel}}.}
%  \item{mc.silent}{if set to \code{TRUE} then all output on \file{stdout} will
%    be suppressed for all parallel processes forked (\file{stderr} is not
%    affected).}
%  \item{mc.cores}{The number of cores to use, i.e.\sspace{}at most how many
%    child processes will be run simultaneously.  Must be at least one,
%    and at least two for parallel operation.  The option is initialized
%    from environment variable \env{MC_CORES} if set.}
%  \item{mc.cleanup}{See the description of this argument in
%    \code{\link{mclapply}}.}
%}
%
%\details{
%  \code{pvec} parallelizes \code{FUN(x, ...)} where \code{FUN} is a
%  function that returns a vector of the same length as
%  \code{x}. \code{FUN} must also be pure (i.e., without side-effects)
%  since side-effects are not collected from the parallel processes. The
%  vector is split into nearly identically sized subvectors on which
%  \code{FUN} is run.  Although it is in principle possible to use
%  functions that are not necessarily maps, the interpretation would be
%  case-specific as the splitting is in theory arbitrary (a warning is
%  given in such cases).
%
%  The major difference between \code{pvec} and \code{\link{mclapply}} is
%  that \code{mclapply} will run \code{FUN} on each element separately
%  whereas \code{pvec} assumes that \code{c(FUN(x[1]), FUN(x[2]))} is
%  equivalent to \code{FUN(x[1:2])} and thus will split into as many
%  calls to \code{FUN} as there are cores (or elements, if fewer), each
%  handling a subset vector.  This makes it more efficient than
%  \code{mclapply} but requires the above assumption on \code{FUN}.
%
%  If \code{mc.cores == 1} this evaluates \code{FUN(v, ...)} in the
%  current process.
%}
%
%\value{
%  The result of the computation -- in a successful case it should be of
%  the same length as \code{v}.  If an error occurred or the function was
%  not a map the result may be shorter or longer, and a warning is given.
%}
%
%\note{
%  Due to the nature of the parallelization, error handling does not
%  follow the usual rules since errors will be returned as strings and
%  results from killed child processes will show up simply as
%  non-existent data.  Therefore it is the responsibility of the user to
%  check the length of the result to make sure it is of the correct size.
%  \code{pvec} raises a warning if that is the case since it does not
%  know whether such an outcome is intentional or not.
%
%  See \code{\link{mcfork}} for the inadvisability of using this with
%  GUI front-ends.
%}
%
%\author{
%  Simon Urbanek and R Core.
%
%  Derived from the \CRANpkg{multicore} package.
%}
%\seealso{
%  \code{\link{mcparallel}}, \code{\link{mclapply}},
%  \code{\link{parLapply}}, \code{\link{clusterMap}}.
%}
%\examples{
%x <- pvec(1:1000, sqrt)
%stopifnot(all(x == sqrt(1:1000)))
%
%\donttest{
%# One use is to convert date strings to unix time in large datasets
%# as that is a relatively slow operation.
%# So let's get some random dates first
%# (A small test only with 2 cores: set options("mc.cores")
%# and increase N for a larger-scale test.)
%N <- 1e5
%dates <- sprintf('\%04d-\%02d-\%02d', as.integer(2000+rnorm(N)),
%                 as.integer(runif(N, 1, 12)), as.integer(runif(N, 1, 28)))
%
%system.time(a <- as.POSIXct(dates))
%
%# But specifying the format is faster
%system.time(a <- as.POSIXct(dates, format = "\%Y-\%m-\%d"))
%
%# pvec ought to be faster, but system overhead can be high
%system.time(b <- pvec(dates, as.POSIXct, format = "\%Y-\%m-\%d"))
%stopifnot(all(a == b))
%
%# using mclapply for this would much slower because each value
%# will require a separate call to as.POSIXct()
%# as lapply(dates, as.POSIXct) does
%system.time(c <- unlist(mclapply(dates, as.POSIXct,  format = "\%Y-\%m-\%d")))
%stopifnot(all(a == c))
%}}
%\keyword{interface}
%
%\encoding{UTF-8}
%\encoding{UTF8}

\encoding{UTF8}

